package com.perf.nio;

import java.io.*;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import com.perf.timer.*;

public class CopyUtil {

	public static void main(String[] args) throws Exception{
		for(int loop=0;loop<10;loop++) {
		CopyUtil.run();
		
		}
		
	}
	public static void run() throws Exception{
		String fromName = "D:\\temp\\Dummy.txt";
		StopWatch sw = new StopWatch();
		
		Thread.sleep(1000);
		sw.start();
		CopyUtil.fileCopyBufferedReader(fromName, fromName + System.currentTimeMillis());
		System.out.println(sw.getElapsedMS());
		
		Thread.sleep(1000);
		sw.start();
		CopyUtil.fileCopyTransfer(fromName, fromName + System.currentTimeMillis());
		System.out.println(sw.getElapsedMS());
		
		Thread.sleep(1000);
		sw.start();
		CopyUtil.fileCopyMapped(fromName, fromName + System.currentTimeMillis());
		System.out.println(sw.getElapsedMS());
		

	}
	public static void fileCopyBufferedReader(String from,String to) throws Exception{
		BufferedReader br=null;
		BufferedWriter bw=null;
		try {
			br=new BufferedReader(new FileReader(from));
			bw=new BufferedWriter(new FileWriter(to));
			String data;
			while((data=br.readLine())!=null) {
				bw.write(data);
				bw.newLine();
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if(bw!=null) bw.close();
			if(br!=null) br.close();
		}
	}
	public static void fileCopyMapped(String from, String to) throws Exception {
		FileInputStream fis=null;
		FileOutputStream fos=null;
		FileChannel in=null;
		FileChannel out=null;
		try {
			fis = new FileInputStream(from);
			fos = new FileOutputStream(to);
			in = fis.getChannel();
			out = fos.getChannel();

			MappedByteBuffer m = in.map(
					FileChannel.MapMode.READ_ONLY, 0, in.size());
			out.write(m);
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if(out!=null) out.close();
			if(in!=null) in.close();
			if(fos!=null) fos.close();
			if(fis!=null) fis.close();
		}
	}

	public static void fileCopyTransfer(String from, String to) throws Exception{
		FileInputStream fis=null;
		FileOutputStream fos=null;
		FileChannel in=null;
		FileChannel out=null;
		try {
			fis = new FileInputStream(from);
			fos = new FileOutputStream(to);
			in = fis.getChannel();
			out = fos.getChannel();
			
			in.transferTo(0, in.size(), out);
			
		} catch (Exception e) {
			e.printStackTrace();
		}finally {
			if(out!=null) out.close();
			if(in!=null) in.close();
			if(fos!=null) fos.close();
			if(fis!=null) fis.close();
		}
	}
}
